package gs.smolban.model

import cats.Show

/** Tickets represent some tracked work.
  *
  * @param id
  *   Unique identifier _within_ the [[Group]].
  * @param group
  *   Unique identifier of the [[Group]] that owns this ticket.
  * @param createdAt
  *   The instant at which this ticket was created.
  * @param createdBy
  *   The unique identifier of the [[User]] who created this ticket.
  * @param title
  *   Arbitrary string title of the ticket.
  * @param description
  *   Markdown contents of the ticket.
  * @param tags
  *   List of [[Tag]] applied to this ticket.
  * @param status
  *   Current [[Status]] of this ticket.
  * @param statusHistory
  *   Linear history of this ticket in terms of status changes.
  */
case class Ticket(
  id: Ticket.Id,
  group: Group.Id,
  createdAt: CreatedAt,
  createdBy: CreatedBy,
  title: String,
  description: String,
  tags: List[Tag],
  status: Status,
  statusHistory: List[Status]
)

object Ticket:

  /** Unique identifier - relative to some [[Group]] - for a [[Ticket]]. This is
    * an opaque type for a Long. In general, [[Ticket]] identifiers are
    * sequences within a group.
    */
  opaque type Id = Long

  object Id:

    /** Instantiate a new [[Ticket.Id]].
      *
      * @param id
      *   The underlying Long.
      * @return
      *   The new [[Ticket.Id]] instance.
      */
    def apply(id: Long): Id = id

    given CanEqual[Id, Id] = CanEqual.derived

    given Show[Id] = _.toLong().toString()

    extension (id: Id)
      /** Unwrap this Ticket ID.
        *
        * @return
        *   The underlying Long value.
        */
      def toLong(): Long = id

  end Id

end Ticket
